@@ -268,9 +268,19 @@ pref("ldap_2.prefs_migrated",      false);
 
 pref("mailnews.confirm.moveFoldersToTrash", true);
 
+pref("mailnews.reuse_message_window", true);
+
 pref("mailnews.start_page.url", "chrome://messenger-region/locale/region.properties");
 pref("mailnews.start_page.enabled", true);
 
+pref("mailnews.remember_selected_message", true);
+
+pref("mail.toolbars.showbutton.file", true);
+pref("mail.toolbars.showbutton.next", true);
+/* not ready yet: pref("mail.toolbars.showbutton.junk", false); */
+pref("mail.toolbars.showbutton.print", true);
+pref("mail.toolbars.showbutton.stop", true);
+
 pref("mailnews.account_central_page.url", "chrome://messenger/locale/messenger.properties");
 
 /* default prefs for Mozilla 5.0 */
@@ -286,14 +296,15 @@ pref("mail.identity.default.stationery_folder","mailbox://nobody@Local%20Folders
 pref("mail.identity.default.directoryServer","");
 pref("mail.identity.default.overrideGlobal_Pref", false);
 
-
 // by default, only collect addresses the user sends to (outgoing)
 // incoming is all spam anyways
 pref("mail.collect_email_address_incoming", false);
 pref("mail.collect_email_address_outgoing", true);
 pref("mail.collect_email_address_newsgroup", false);
-pref("mail.collect_email_address_enable_size_limit", true);
-pref("mail.collect_email_address_size_limit", 700);
+
+// by default, use the Personal Addressbook for collection
+pref("mail.collect_addressbook","moz-abmdbdirectory://abook.mab"); // the Personal addressbook.
+
 pref("mail.default_sendlater_uri","mailbox://nobody@Local%20Folders/Unsent%20Messages");
 
 pref("mail.server.default.port", -1);
@@ -330,6 +341,22 @@ pref("mail.server.default.allows_specialfolders_usage", true);
 pref("mail.server.default.canCreateFolders", true);
 pref("mail.server.default.canFileMessages", true);
 
+// special enhancements for IMAP servers
+pref("mail.server.default.store_read_mail_in_pfc", false);  
+pref("mail.server.default.store_sent_mail_in_pfc", false);  
+
+// for spam
+pref("mail.server.default.spamLevel",0);  // 0 off, 100 on.  not doing bool since we might have real levels one day.
+pref("mail.server.default.moveOnSpam",false);
+pref("mail.server.default.moveTargetMode",0); // 0 == "Junk" on server, 1 == specific folder
+pref("mail.server.default.spamActionTargetAccount","");
+pref("mail.server.default.spamActionTargetFolder","");
+pref("mail.server.default.useWhiteList",true);
+pref("mail.server.default.whiteListAbURI","moz-abmdbdirectory://abook.mab");  // the Personal addressbook.
+pref("mail.server.default.purgeSpam",false);
+pref("mail.server.default.purgeSpamInterval",14); // 14 days
+pref("mail.server.default.spamLoggingEnabled",false);
+
 pref("mail.smtpserver.default.auth_method", 1); // auth any
 pref("mail.smtpserver.default.try_ssl", 0);
 
@@ -389,6 +416,11 @@ pref("mailnews.append_preconfig_accounts.version",1);
 pref("mail.append_preconfig_smtpservers.version",1);
 
 pref("mail.biff.play_sound",true);
+// 0 == default system sound, 1 == user specified wav
+pref("mail.biff.play_sound.type",0);
+// _moz_mailbeep is a magic key, for the default sound.
+// otherwise, this needs to be a file url
+pref("mail.biff.play_sound.url","");
 pref("mail.biff.show_alert", true);
 
 pref("mail.content_disposition_type", 0);
