@@ -22,6 +22,7 @@
  *               Hkan Waara <hwaara@chello.se>
  *               Jan Varga <varga@utcru.sk>
  *               Seth Spitzer <sspitzer@netscape.com>
+ *               David Bienvenu <bienvenu@netscape.com>
  */
 
 var gMessengerBundle;
@@ -32,6 +33,7 @@ var gOfflineManager;
 var gWindowManagerInterface;
 var gPrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
 var gPrintSettings = null;
+var gWindowReuse  = 0;
 
 var gTimelineService = null;
 var gTimelineEnabled = ("@mozilla.org;timeline-service;1" in Components.classes);
@@ -49,11 +51,19 @@ if (gTimelineEnabled) {
   }
 }
 
+var disallow_classes_no_html = 1; /* the user preference,
+     if HTML is not allowed. I assume, that the user could have set this to a
+     value > 1 in his prefs.js or user.js, but that the value will not
+     change during runtime other than through the MsgBody*() functions below.*/
+
 // Disable the new account menu item if the account preference is locked.
 // Two other affected areas are the account central and the account manager
 // dialog.
 function menu_new_init()
 {
+  if (!gMessengerBundle)
+    gMessengerBundle = document.getElementById("bundle_messenger");
+
   if (!nsPrefBranch) {
     var prefService = Components.classes["@mozilla.org/preferences-service;1"];
     prefService = prefService.getService();
@@ -63,6 +73,31 @@ function menu_new_init()
   var newAccountItem = document.getElementById('newAccountMenuItem');
   if (nsPrefBranch.prefIsLocked("mail.disable_new_account_addition"))
     newAccountItem.setAttribute("disabled","true");
+
+  // Change "New Folder..." menu according to the context
+  var folderArray = GetSelectedMsgFolders();
+  if (folderArray.length == 0)
+    return;
+  var msgFolder = folderArray[0];
+  var isServer = msgFolder.isServer;
+  var serverType = msgFolder.server.type;
+  var canCreateNew = msgFolder.canCreateSubfolders;
+  var isInbox = IsSpecialFolder(msgFolder, MSG_FOLDER_FLAG_INBOX);
+  var isIMAPFolder = serverType == "imap";
+  var ioService = Components.classes["@mozilla.org/network/io-service;1"]
+                         .getService(Components.interfaces.nsIIOService);
+  var showNew = ((serverType != 'nntp') && canCreateNew) || isInbox;
+  ShowMenuItem("menu_newFolder", showNew);
+  EnableMenuItem("menu_newFolder", !isIMAPFolder || !ioService.offline);
+  if (showNew)
+  {
+    if (isServer || isInbox)
+      SetMenuItemLabel("menu_NewFolder",
+        gMessengerBundle.getString("newFolder"));
+    else
+      SetMenuItemLabel("menu_NewFolder",
+        gMessengerBundle.getString("newSubfolder"));
+  }
 }
 
 function goUpdateMailMenuItems(commandset)
@@ -163,6 +198,7 @@ function InitViewSortByMenu()
     setSortByMenuItemCheckState("sortByThreadMenuitem", (sortType == nsMsgViewSortType.byThread));
     setSortByMenuItemCheckState("sortByUnreadMenuitem", (sortType == nsMsgViewSortType.byUnread));
     setSortByMenuItemCheckState("sortByLabelMenuitem", (sortType == nsMsgViewSortType.byLabel));
+    setSortByMenuItemCheckState("sortByJunkStatusMenuitem", (sortType == nsMsgViewSortType.byJunkStatus));
  
     // the Sender / Recipient menu is dynamic
     setSortByMenuItemCheckState("sortBySenderOrRecipientMenuitem", (sortType == nsMsgViewSortType.byAuthor) || (sortType == nsMsgViewSortType.byRecipient));
@@ -256,10 +292,15 @@ function InitMessageMenu()
       watchThreadMenuItem.setAttribute("hidden", isNews ? "" : "true");
   }
 
-  //disable the move and copy menus only if there are no messages selected.
+  // disable the move and copy menus if there are no messages selected.
+  // disable the move menu if we can't delete msgs from the folder
   var moveMenu = document.getElementById("moveMenu");
   if(moveMenu)
-      moveMenu.setAttribute("disabled", !aMessage);
+  {
+      var msgFolder = GetLoadedMsgFolder();
+      var enableMenuItem = aMessage && msgFolder && msgFolder.canDeleteMessages;
+      moveMenu.setAttribute("disabled", !enableMenuItem);
+  }
 
   var copyMenu = document.getElementById("copyMenu");
   if(copyMenu)
@@ -297,16 +338,69 @@ function InitViewHeadersMenu()
     menuitem.setAttribute("checked", "true"); 
 }
 
+function InitViewBodyMenu()
+{
+  var html_as = 0;
+  var prefer_plaintext = false;
+  var disallow_classes = 0;
+  try
+  {
+    prefer_plaintext = pref.getBoolPref("mailnews.display.prefer_plaintext");
+    html_as = pref.getIntPref("mailnews.display.html_as");
+    disallow_classes =
+                    pref.getIntPref("mailnews.display.disallow_mime_handlers");
+    if (disallow_classes > 0)
+      disallow_classes_no_html = disallow_classes;
+    // else disallow_classes_no_html keeps its inital value (see top)
+  }
+  catch (ex)
+  {
+    dump("failed to get the body plaintext vs. HTML prefs\n");
+  }
+
+  var AllowHTML_checked = false;
+  var Sanitized_checked = false;
+  var AsPlaintext_checked = false;
+  if (prefer_plaintext == false && html_as == 0 && disallow_classes == 0)
+    AllowHTML_checked = true;
+  else if (prefer_plaintext == false && html_as == 3 && disallow_classes > 0)
+    Sanitized_checked = true;
+  else if (prefer_plaintext == true && html_as == 1 && disallow_classes > 0)
+    AsPlaintext_checked = true;
+  // else (the user edited prefs/user.js) check none of the radio menu items
+
+  var AllowHTML_menuitem = document.getElementById("bodyAllowHTML");
+  var Sanitized_menuitem = document.getElementById("bodySanitized");
+  var AsPlaintext_menuitem = document.getElementById("bodyAsPlaintext");
+  if (AllowHTML_menuitem
+      && Sanitized_menuitem
+      && AsPlaintext_menuitem)
+  {
+    AllowHTML_menuitem.setAttribute("checked",
+                                    AllowHTML_checked ? "true" : "false");
+    Sanitized_menuitem.setAttribute("checked",
+                                      Sanitized_checked ? "true" : "false");
+    AsPlaintext_menuitem.setAttribute("checked",
+                                      AsPlaintext_checked ? "true" : "false");
+  }
+  else
+    dump("Where is my View|Body menu?\n");
+}
+
 function IsNewsMessage(messageUri)
 {
-    if (!messageUri) return false;
-    return (messageUri.substring(0,14) == "news-message:/")
+    if (!messageUri) 
+      return false;
+    else 
+      return (messageUri.substring(0,14) == "news-message:/");
 }
 
 function IsImapMessage(messageUri)
 {
-    if (!messageUri) return false;
-    return (messageUri.substring(0,14) == "imap-message:/")
+    if (!messageUri) 
+      return false;
+    else
+      return (messageUri.substring(0,14) == "imap-message:/");
 }
 
 function SetMenuItemLabel(menuItemId, customLabel)
@@ -440,6 +534,19 @@ function SelectedMessagesAreDeleted()
     }
 }
 
+function SelectedMessagesAreJunk()
+{
+    var isJunk;
+    try {
+        var junkScore = gDBView.hdrForFirstSelectedMessage.getStringProperty("junkscore");
+        isJunk =  ((junkScore != "") && (junkScore != "0"));
+    }
+    catch (ex) {
+        isJunk = false;
+    }
+    return isJunk;
+}
+
 function SelectedMessagesAreRead()
 {
     var isRead;
@@ -485,7 +592,7 @@ function GetWindowMediator()
     if (gWindowManagerInterface)
         return gWindowManagerInterface;
 
-    var windowManager = Components.classes['@mozilla.org/rdf/datasource;1?name=window-mediator'].getService();
+    var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
     return (gWindowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator));
 }
 
@@ -552,7 +659,7 @@ function MsgGetMessagesForAllServers(defaultServer)
             {
                 if (defaultServer && defaultServer.equals(currentServer))
                 {
-                    dump(currentServer.serverURI + "...skipping, already opened\n");
+                    //dump(currentServer.serverURI + "...skipping, already opened\n");
                 }
                 else
                 {
@@ -761,8 +868,11 @@ function MsgForwardMessage(event)
 {
   var forwardType = 0;
   try {
-      forwardType = gPrefs.getIntPref("mail.forward_message_mode");
-  } catch (e) {dump ("failed to retrieve pref mail.forward_message_mode");}
+    forwardType = gPrefs.getIntPref("mail.forward_message_mode");
+  }
+  catch (ex) {
+    dump("failed to retrieve pref mail.forward_message_mode");
+  }
 
   // mail.forward_message_mode could be 1, if the user migrated from 4.x
   // 1 (forward as quoted) is obsolete, so we treat is as forward inline
@@ -975,11 +1085,78 @@ function MsgOpenSelectedMessages()
 
   var indices = GetSelectedIndices(dbView);
   var numMessages = indices.length;
-  for (var i = 0; i < numMessages; i++) {
-    MsgOpenNewWindowForMessage(dbView.getURIForViewIndex(indices[i]),dbView.getFolderForViewIndex(indices[i]).URI);
+
+  gWindowReuse = gPrefs.getBoolPref("mailnews.reuse_message_window");
+  // This is a radio type button pref, currently with only 2 buttons.
+  // We need to keep the pref type as 'bool' for backwards compatibility
+  // with 4.x migrated prefs.  For future radio button(s), please use another
+  // pref (either 'bool' or 'int' type) to describe it.
+  //
+  // gWindowReuse values: false, true
+  //    false: open new standalone message window for each message
+  //    true : reuse existing standalone message window for each message
+  if ((gWindowReuse) && (numMessages == 1)) {
+      if (!MsgOpenExistingWindowForMessage(dbView.getURIForViewIndex(indices[0]))) {
+          MsgOpenNewWindowForMessage(dbView.getURIForViewIndex(indices[0]),dbView.getFolderForViewIndex(indices[0]).URI);
+      }
+  } else {
+      for (var i = 0; i < numMessages; i++) {
+          MsgOpenNewWindowForMessage(dbView.getURIForViewIndex(indices[i]),dbView.getFolderForViewIndex(indices[i]).URI);
+      }
   }
 }
 
+function MsgOpenExistingWindowForMessage(aMessageUri)
+{
+    var messageUri;
+    var msgHdr = null;
+    var windowID = GetWindowByWindowType("mail:messageWindow");
+
+    if(!windowID)
+      return false;
+
+    if (!aMessageUri) {
+        var currentIndex = gDBView.QueryInterface(Components.interfaces.nsIOutlinerView).outlinerView.selection;
+        messageUri = gDBView.getURIForViewIndex(currentIndex);
+    }
+    else
+        messageUri = aMessageUri;
+
+    // be sure to pass in the current view....
+    if (!messageUri)
+        return false;
+
+    try {
+        msgHdr = messenger.messageServiceFromURI(messageUri).messageURIToMsgHdr(messageUri);
+        if (!msgHdr)
+            return false;
+
+        if (msgHdr.folder.URI != windowID.gCurrentFolderUri) {
+            if ("CreateView" in windowID) {
+                // Reset the window's message uri and folder uri vars, and
+                // update the command handlers to what's going to be used.
+                // This has to be done before the call to CreateView().
+                windowID.gCurrentMessageUri = messageUri;
+                windowID.gCurrentFolderUri = msgHdr.folder.URI;
+                windowID.UpdateMailToolbar('MsgOpenExistingWindowForMessage');
+                windowID.CreateView(gDBView);
+            }
+            else
+                return false;
+        }
+
+        windowID.gDBView.loadMessageByMsgKey(msgHdr.messageKey);
+    }
+    catch (ex) {
+        dump("reusing existing standalone message window failed: " + ex + "\n");
+        return false;
+    }
+    // bring existing window to front
+    windowID.focus();
+
+    return true;
+}
+
 function MsgOpenNewWindowForMessage(messageUri, folderUri)
 {
     if (!messageUri)
@@ -1012,6 +1189,11 @@ function CloseMailWindow()
     window.close();
 }
 
+function MsgJunk()
+{
+  JunkSelectedMessages(!SelectedMessagesAreJunk());
+}
+
 function MsgMarkMsgAsRead(markRead)
 {
     if (!markRead) {
@@ -1066,10 +1248,10 @@ function MsgFind()
   findInPage(getMessageBrowser(), contentWindow, contentWindow)
 }
 
-function MsgFindAgain()
+function MsgFindAgain(reverse)
 {
   var contentWindow = window.top._content;
-  findAgainInPage(getMessageBrowser(), contentWindow, contentWindow)
+  findAgainInPage(getMessageBrowser(), contentWindow, contentWindow, reverse)
 }
 
 function MsgCanFindAgain()
@@ -1083,7 +1265,8 @@ function MsgFilters(emailAddress)
     var args;
     if (emailAddress)
     {
-      /* we have to do prefill filter so we are going to launch the filterEditor dialog
+      /* we have to do prefill filter so we are going to 
+         launch the filterEditor dialog
          and prefill that with the emailAddress */
          
       var curFilterList = preselectedFolder.getFilterList(msgWindow);
@@ -1092,24 +1275,54 @@ function MsgFilters(emailAddress)
       window.openDialog("chrome://messenger/content/FilterEditor.xul", "", 
                         "chrome, modal, resizable,centerscreen,dialog=yes", args);
 
-      /* if the user hits ok in the filterEditor dialog we set args.refresh=true there
+      /* if the user hits ok in the filterEditor dialog we set 
+         args.refresh=true there
          we check this here in args to show filterList dialog */
-
       if ("refresh" in args && args.refresh)
       {
-         args = { folder: preselectedFolder };
-    window.openDialog("chrome://messenger/content/FilterListDialog.xul", "", 
-                        "chrome,modal,resizable,centerscreen,dialog=yes", args);
-}
+         args = { refresh: true, folder: preselectedFolder };
+         MsgFilterList(args);
+      }
     }
-    else  //just launch filterList dialog
+    else  // just launch filterList dialog
     {
-      args = { folder: preselectedFolder };
-      window.openDialog("chrome://messenger/content/FilterListDialog.xul", "", 
-                       "chrome,modal,resizable,centerscreen,dialog=yes", args);
+      args = { refresh: false, folder: preselectedFolder };
+      MsgFilterList(args);
     }
 }
 
+function MsgApplyFilters()
+{
+  var filterService = Components.classes["@mozilla.org/messenger/services/filters;1"].getService(Components.interfaces.nsIMsgFilterService);
+
+  var preselectedFolder = GetFirstSelectedMsgFolder();
+  var selectedFolders = Components.classes["@mozilla.org/supports-array;1"].createInstance(Components.interfaces.nsISupportsArray);
+  selectedFolders.AppendElement(preselectedFolder);
+         
+  var curFilterList = preselectedFolder.getFilterList(msgWindow);
+  // create a new filter list and copy over the enabled filters to it.
+  // We do this instead of having the filter after the fact code ignore
+  // disabled filters because the Filter Dialog filter after the fact
+  // code would have to clone filters to allow disabled filters to run,
+  // and we don't support cloning filters currently.
+  var tempFilterList = filterService.getTempFilterList(preselectedFolder);
+  var numFilters = curFilterList.filterCount;
+  // make sure the temp filter list uses the same log stream
+  tempFilterList.logStream = curFilterList.logStream;
+  tempFilterList.loggingEnabled = curFilterList.loggingEnabled;
+  var newFilterIndex = 0;
+  for (var i = 0; i < numFilters; i++)
+  {
+    var curFilter = curFilterList.getFilterAt(i);
+    if (curFilter.enabled)
+    {
+      tempFilterList.insertFilterAt(newFilterIndex, curFilter);
+      newFilterIndex++;
+    }
+  }
+  filterService.applyFiltersToFolders(tempFilterList, selectedFolders, msgWindow);
+}
+
 function MsgViewAllHeaders()
 {
     gPrefs.setIntPref("mail.show_headers",2);
@@ -1131,6 +1344,35 @@ function MsgViewBriefHeaders()
     return true;
 }
 
+function MsgBodyAllowHTML()
+{
+    gPrefs.setBoolPref("mailnews.display.prefer_plaintext", false);
+    gPrefs.setIntPref("mailnews.display.html_as", 0);
+    gPrefs.setIntPref("mailnews.display.disallow_mime_handlers", 0);
+    MsgReload();
+    return true;
+}
+
+function MsgBodySanitized()
+{
+    gPrefs.setBoolPref("mailnews.display.prefer_plaintext", false);
+    gPrefs.setIntPref("mailnews.display.html_as", 3);
+    gPrefs.setIntPref("mailnews.display.disallow_mime_handlers",
+                      disallow_classes_no_html);
+    MsgReload();
+    return true;
+}
+
+function MsgBodyAsPlaintext()
+{
+    gPrefs.setBoolPref("mailnews.display.prefer_plaintext", true);
+    gPrefs.setIntPref("mailnews.display.html_as", 1);
+    gPrefs.setIntPref("mailnews.display.disallow_mime_handlers",
+                      disallow_classes_no_html);
+    MsgReload();
+    return true;
+}
+
 function MsgReload()
 {
     ReloadMessage();
@@ -1190,11 +1432,8 @@ function GetPrintSettings()
   return gPrintSettings;
 }
 
-function PrintEnginePrint()
+function PrintEnginePrintInternal(messageList, numMessages, doPrintPreview, msgType)
 {
-    var messageList = GetSelectedMessages();
-    var numMessages = messageList.length;
-
     if (numMessages == 0) {
         dump("PrintEnginePrint(): No messages selected.\n");
         return false;
@@ -1203,12 +1442,24 @@ function PrintEnginePrint()
     if (gPrintSettings == null) {
       gPrintSettings = GetPrintSettings();
     }
-
     printEngineWindow = window.openDialog("chrome://messenger/content/msgPrintEngine.xul",
                                           "",
                                           "chrome,dialog=no,all,centerscreen",
-                                          numMessages, messageList, statusFeedback, gPrintSettings);
+                                          numMessages, messageList, statusFeedback, gPrintSettings, doPrintPreview, msgType, window);
     return true;
+
+}
+
+function PrintEnginePrint()
+{
+    var messageList = GetSelectedMessages();
+    return PrintEnginePrintInternal(messageList, messageList.length, false, Components.interfaces.nsIMsgPrintEngine.MNAB_PRINT_MSG);
+}
+
+function PrintEnginePrintPreview()
+{
+    var messageList = GetSelectedMessages();
+    return PrintEnginePrintInternal(messageList, 1, true, Components.interfaces.nsIMsgPrintEngine.MNAB_PRINTPREVIEW_MSG);
 }
 
 function IsMailFolderSelected()
@@ -1315,7 +1566,8 @@ var gMarkButton = null;
 
 function SetUpToolbarButtons(uri)
 {
-    ///dump("XXX SetUpToolbarButtons("+uri+")\n");
+    // dump("SetUpToolbarButtons("+uri+")\n");
+
     // eventually, we might want to set up the toolbar differently for imap,
     // pop, and news.  for now, just tweak it based on if it is news or not.
     var forNews = isNewsURI(uri);
@@ -1360,7 +1612,7 @@ function getMarkupDocumentViewer()
 
 function MsgSynchronizeOffline()
 {
-    dump("in MsgSynchronize() \n"); 
+    //dump("in MsgSynchronize() \n"); 
     window.openDialog("chrome://messenger/content/msgSynchronize.xul",
           "", "centerscreen,chrome,modal,titlebar,resizable=yes",{msgWindow:msgWindow}); 		     
 }
@@ -1535,6 +1787,7 @@ function CommandUpdate_UndoRedo()
 function SetupUndoRedoCommand(command)
 {
     var loadedFolder = GetLoadedMsgFolder();
+
     // if we have selected a server, and are viewing account central
     // there is no loaded folder
     if (!loadedFolder)
@@ -1603,7 +1856,11 @@ function OnMsgLoaded(folder, msgURI)
       var messageURI = GetLoadedMessage();
       if (messageURI != msgURI)
       {
-        dump("not loading msg into this window - loaded message = " + messageURI + "loading " + msgURI + "\n");
+//        XXX TODO
+//        bienvenu tells me:  
+//        if you have two message windows open, you can get multiple attempts
+//        to copy into the pfc.  the second will fail and assert.
+          dump("not loading msg into this window - loaded message = " + messageURI + "loading " + msgURI + "\n");
 //        return;
       }
       var msgHdr = messenger.messageServiceFromURI(messageURI).messageURIToMsgHdr(messageURI);
@@ -1623,7 +1880,7 @@ function OnMsgLoaded(folder, msgURI)
       }
       if (copyToOfflineFolder)
       {
-		    var messages = Components.classes["@mozilla.org/supports-array;1"].createInstance(Components.interfaces.nsISupportsArray);
+        var messages = Components.classes["@mozilla.org/supports-array;1"].createInstance(Components.interfaces.nsISupportsArray);
         messages.AppendElement(msgHdr);
 
         res = outputPFC.copyMessages(currentMsgFolder, messages, false /*isMove*/, msgWindow /* nsIMsgWindow */, null /* listener */, false /* isFolder */, false /*allowUndo*/ );
@@ -1631,3 +1888,53 @@ function OnMsgLoaded(folder, msgURI)
      }
 }
 
+function MsgSearchMessages()
+{
+  var preselectedFolder = null;
+  if ("GetFirstSelectedMsgFolder" in window)
+    preselectedFolder = GetFirstSelectedMsgFolder();
+
+  var args = { folder: preselectedFolder };
+  OpenOrFocusWindow(args, "mailnews:search", "chrome://messenger/content/SearchDialog.xul");
+}
+
+function MsgJunkMail()
+{
+  var preselectedFolder = null;
+  if ("GetFirstSelectedMsgFolder" in window)
+    preselectedFolder = GetFirstSelectedMsgFolder();
+
+  var args = { folder: preselectedFolder };
+  OpenOrFocusWindow(args, "mailnews:junk", "chrome://messenger/content/junkMail.xul");
+}
+
+function MsgSearchAddresses()
+{
+  var args = { directory: null };
+  OpenOrFocusWindow(args, "mailnews:absearch", "chrome://messenger/content/ABSearchDialog.xul");
+}
+ 
+function MsgFilterList(args)
+{
+  OpenOrFocusWindow(args, "mailnews:filterlist", "chrome://messenger/content/FilterListDialog.xul");
+}
+
+function GetWindowByWindowType(windowType)
+{
+  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
+  return windowManager.getMostRecentWindow(windowType);
+}
+
+function OpenOrFocusWindow(args, windowType, chromeURL)
+{
+  var desiredWindow = GetWindowByWindowType(windowType);
+
+  if (desiredWindow) {
+    desiredWindow.focus();
+    if ("refresh" in args && args.refresh)
+      desiredWindow.refresh();
+  }
+  else
+    window.openDialog(chromeURL, "", "chrome,resizable,status,centerscreen,dialog=no", args);
+}
+
